%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Generator of random QP in the file 'randqp1.dat'.
%
%  Author: J.Ch. Gilbert, INRIA, November 2012
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------
% Initialization
%---------------

  clear all;

%-----------------------
% Set the data file name
%-----------------------

  datafile = 'xxx';
  fmat = fopen(datafile,'w');
  if fmat < 0
    fprintf ('\n### Cannot open file ''%s'' for writing\n\n',datafile);
    return
  end

%------------------------
% General initializations
%------------------------

  reset(RandStream.getDefaultStream);	% reset the default random stream to its initial state

%---------------------------------------------
% Generate the data and write them on the file
%---------------------------------------------

  n = 6;
  H = rand(n,n); L = tril(H); H=L*L';
  eigH = eig(H);
  eigHmin = min(eigH);
  eigHmax = max(eigH);
  fprintf('\nEigenvalues of H: min = %9.2e, max = %8.2e, cond = %8.2e\n',eigHmin,eigHmax,eigHmax/eigHmin);
  while (eigHmin < 0) | (eigHmax/eigHmin > 1.e10)
    L = L+eye(n); H=L*L';
    eigH = eig(H);
    eigHmin = min(eigH);
    eigHmax = max(eigH);
    fprintf('Eigenvalues of H: min = %9.2e, max = %8.2e\n',eigHmin,eigHmax);
  end
  g = rand(n,1);

  fwrite(fmat,n,'int32');	% 'integer*4' in C or Fortran, integer on 32 bits
  fwrite(fmat,g,'double');	% 'real*8' in C or Fortran, floating point on 64 bits
  fwrite(fmat,H,'double');

  return
