%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Example of Matlab program using QPAL as an optimization solver.
%
%  The program is executed by typing
%
%      test
%
%  in the Matlab window.
%
%  QPAL can solve a minimization problem of the form
%
%      minimize     g'*x + 0.5*x'*H*x
%
%  Author: J.Ch. Gilbert, INRIA, November 2012
%   modified by Corentin
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------
% Initialization
%---------------

  clf;
  clear all;
  format compact
  format long

%------------------------
% General initializations
%------------------------

  addpath '../src'          % Add the QPAL directory to the Matlab path
  %reset(RandStream.getDefaultStream);   % reset the default random stream to its initial state

%--------------------------
% Loop on the test problems
%--------------------------

% Read the data on 'randqp1.dat'

fmat = fopen('data1000.dat','r');
n = fread(fmat,1,'int32');
g = fread(fmat,n,'double');
H = fread(fmat,[n,n],'double');
fclose(fmat);
% Set the options of QPAL
  
options.fout = 1;
options.qtol = 1.e-15;
options.verb = 3;

options.gtol = 1.e-8;
options.alit = 100;
options.cgit = Inf;
options.mvpd = Inf;

% solve the QP by QPAL

fprintf('\nRunning qpal\n');

[x1,info] = qpal(g,H,[],[],[],options);
%x1

fprintf('\n=========================================\n');
try
    OCTAVE_VERSION;
    %Octave
    time_init = clock;
    fprintf('\nRunning Octave qp\n');
    xOct = qp([], H,g, [],[], [],[]);
    time = etime(clock,time_init);
    fprintf('Cpu time for Octave:   %f seconds \n',time);
    %xOct
catch
    time_init = clock;
    fprintf('\nRunning Matlab quadprog\n');
    qp_options = optimset('LargeScale','off','Display','iter','TolFun',1e-8,'TolX',1e-10);
    [xMat,f,exitflag] = quadprog(H,g,[],[],[],[],[],[],[],qp_options);
    time = etime(clock,time_init);
    fprintf('Cpu time for Matlab:   %f seconds \n',time);
    %xMat
end





return
