%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Temporary file that transform the data from one format to the new one
%  (sparse vectors).
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------
% Initialization
%---------------

  clf;
  clear all;
  format compact
  format long

%--------------------------------------
% Choose the file of the example to run
%--------------------------------------

% datafile = 'solved2a.tdt';		%+0 2 bounds, solved at the inital point 0
% datafile = 'solved2b.tdt';		%+1 2 inequality constraints, solved at the initial point 0 (must compute multiplier)

% datafile = 'special3a.tdt';		%+0 used to test some part of Qpalm, failure of Quadprog
% datafile = 'special3b.tdt';		%+1 used to test some part of Qpalm, linear problem
% datafile = 'special3c.tdt';		%+0 used in the COPI'14 conference and the MPC paper, failure of Quadprog
% datafile = 'special4a.tdt';		%+0 another failure of Quadprog

% datafile = 'randqp10a.tdt';		%+0 unbounded problem (nullspace // e1+e2), no constraint
% datafile = 'randqp10b.tdt';		%+0 unbounded problem (nullspace // e1+e2), bound constraints
% datafile = 'randqp10c.tdt';		%+0 (nullspace // e1+e2), bound constraints, one solution
% datafile = 'randqp10d.tdt';		%+1 equality constraints, with a solution
% datafile = 'randqp10e.tdt';		%+1 equality constraints, unbounded if feasible
% datafile = 'randqp10f.tdt';		%+3(car-teste-les--contraintes-translatees-apres-3it) equality constraints, infeasible problem
% datafile = 'randqp10g.tdt';		%+1 equality constraints and bounds on x, unbounded problem
% datafile = 'randqp10h.tdt';		%+1 equality constraints and bounds on x, with a solution
  datafile = 'randqp10i.tdt';		%+1 equality, inequality+equality constraints, bounds on x, with a solution
% datafile = 'randqp10j.tdt';		% equality, inequality+equality constraints, bounds on x, unbounded if feasible
% datafile = 'randqp10k.tdt';		%+3(car-teste-les--contraintes-translatees-apres-3it) equality, inequality+equality constraints, bounds on x, infeasible
% datafile = 'randqp10l.tdt';		%+1 equality, inequality+equality constraints, with a solution
% datafile = 'randqp10m.tdt';		% no constraint, nullspace = vect{e5,e7}, unbounded along e5 and e7
% datafile = 'randqp10n.tdt';		% no constraint, nullspace = vect{e5,e7}, unbounded along e7
% datafile = 'randqp10o.tdt';		% no constraint, nullspace = vect{e5,e7}, with solution
% datafile = 'randqp10p.tdt';		% bound constraint on x5, nullspace = vect{e5,e7}, unbounded along e7
% datafile = 'randqp10q.tdt';		% inequality constraint on x4+x5, nullspace = vect{e5,e7}, unbounded along e7
% datafile = 'randqp10r.tdt';		% nonconvex unconstrained problem

% datafile = 'cutest_aug3d_dense.tdt';	%
% datafile = 'cutest_primalc1.tdt';	%+1
% datafile = 'cutest_qpcboe12.tdt';	%+3
% datafile = 'cutest_qscagr7.tdt';	%+3
% datafile = 'cutest_qscfxm1.tdt';	%+1 Gros calculs avec Qpalm. check5b and check1 done; error in qpalm_cholp
% datafile = 'cutest_values.tdt';	%+1

%------------------------
% General initializations
%------------------------

  addpath '../src'			% Add the QPALM directory to the Matlab path

%--------------------------------------------
% Some paramaters and choice of the QP solver
%--------------------------------------------

% fid = fopen('res','w');
  fid = 1;

%---------------
% Open datafiles
%---------------

  fbin = fopen('xxx.bdt','w');
  if fbin < 0
    fprintf ('\n### Cannot open file ''%s'' for writing\n\n','xxx.bdt');
    return
  end
  ftxt = fopen('xxx.tdt','w');
  if ftxt < 0
    fprintf ('\n### Cannot open file ''%s'' for writing\n\n','xxx.tdt');
    return
  end

%--------------
% Read the data
%--------------

  fprintf(fid,'\n*** Problem ''%s'' ***\n\n',datafile);

  % definition du POQ
  %
  %   min   g'*x + 0.5*x'*H*x
  %   s.t.  lbx <=    x <= ubx
  %         lbi <= AI*x <= ubi
  %         AE*x = be

  % Read data on the 'datafile' (the format of the datafile is revealed by the way it is read below)

  fdat = fopen(datafile,'r');
  if fdat < 0
    fprintf ('\n### Cannot open file ''%s''\n\n',datafile);
    return
  end

  % dimension
  % ^^^^^^^^^

  n = sscanf(fgets(fdat),'%i',1);		% nb of variables

  fwrite(fbin,n,'int32');	% 'integer*4' in C or Fortran, integer on 32 bits
  fprintf(ftxt,'%0i		variables\n',n);

  % Gradient at the origin
  % ^^^^^^^^^^^^^^^^^^^^^^

  g = fscanf(fdat,'%e',n);
  g = g(:);
  [i,j,v] = find(sparse(g));
  nnz = length(i);

  if nnz < 0.8*n

    fwrite(fbin,nnz,'int32');	% g is sparse
    fprintf(ftxt,'%0i		nonzero elements in the sparse gradient, given below by their (index, value)\n',nnz);

    for k = 1:nnz
      fwrite(fbin,i(k),'int32');
      fwrite(fbin,v(k),'double');
      fprintf(ftxt,'%i  %23.16e\n',i(k),v(k));
    end

  else

    fwrite(fbin,-1,'int32');	% g is dense
    fprintf(ftxt,'%0i		the gradient is dense, with the following elements\n',-1);

    fwrite(fbin,g,'double');	% 'real*8' in C or Fortran, floating point on 64 bits
    fprintf(ftxt,'%23.16e\n',g);

  end

  % Hessian
  % ^^^^^^^

  fgets(fdat);					% next line
  nnz = sscanf(fgets(fdat),'%i',1);		% type of Hessian
  if nnz == 0					% H is full
    H = fscanf(fdat,'%e',[n,n]);
  elseif nnz > 0				% H is sparse
    for k = 1:nnz
      i(k) = fscanf(fdat,'%i',1);
      j(k) = fscanf(fdat,'%i',1);
      v(k) = fscanf(fdat,'%e',1);
    end
    H = sparse(i,j,v,n,n);
  else
    fprintf ('\n### nnz (=%i) for H should be >= 0\n\n',nnz);
    return
  end
  fgets(fdat);					% next line

  [i,j,v] = find(sparse(H));
  nnz = length(i);

  if nnz < 0.8*n*n

    fwrite(fbin,nnz,'int32');	% H is sparse
    fprintf(ftxt,'%0i		nonzero elements in the sparse Hessian H, given below by their (row, column, value)\n',nnz);

    for k = 1:nnz
      fwrite(fbin,i(k),'int32');
      fwrite(fbin,j(k),'int32');
      fwrite(fbin,v(k),'double');
      fprintf(ftxt,'%i  %i  %23.16e\n',i(k),j(k),v(k));
    end

  else

    fwrite(fbin,-1,'int32');	% H is dense
    fprintf(ftxt,'%0i		the Hessian is dense, with the following elements (ranged column by column)\n',-1);

    fwrite(fbin,H,'double');
    fprintf(ftxt,'%23.16e\n',H);

  end

  % Bounds on x
  % ^^^^^^^^^^^

  nlbx = sscanf(fgets(fdat),'%i',1);		% nb of lower bounds on x
  fwrite(fbin,nlbx,'int32');
  fprintf(ftxt,'%0i		lower bounds on x',nlbx);

  if nlbx
    fprintf(ftxt,', followed by their indices and values\n');
    for i = 1:nlbx
      j = fscanf(fdat,'%i',1);
      v = fscanf(fdat,'%e',1);
      fwrite(fbin,j,'int32');
      fwrite(fbin,v,'double');
      fprintf(ftxt,'%0i  %23.16e\n',j,v);
    end
    fgets(fdat);				% next line
  else
    fprintf(ftxt,'\n');
  end

  nubx = sscanf(fgets(fdat),'%i',1);		% nb of upper bounds on x
  fwrite(fbin,nubx,'int32');
  fprintf(ftxt,'%0i		upper bounds on x',nubx);

  if nubx
    fprintf(ftxt,', followed by their indices and values\n');
    for i = 1:nubx
      j = fscanf(fdat,'%i',1);
      v = fscanf(fdat,'%e',1);
      fwrite(fbin,j,'int32');
      fwrite(fbin,v,'double');
      fprintf(ftxt,'%0i  %23.16e\n',j,v);
    end
    fgets(fdat);				% next line
  else
    fprintf(ftxt,'\n');
  end

  % Inequality constraints
  % ^^^^^^^^^^^^^^^^^^^^^^

  mi = fscanf(fdat,'%i',1);
  fwrite(fbin,mi,'int32');
  fprintf(ftxt,'%0i		inequality constraints\n',mi);
  fgets(fdat);					% next line to skip the comment or to see whether the eof is reached

  if mi

    nnz = sscanf(fgets(fdat),'%i',1);		% type of Jacobian
    if nnz == 0					% AI is full
      AI = fscanf(fdat,'%e',[mi,n]);
    elseif nnz > 0				% AI is sparse
      for k = 1:nnz
        i(k) = fscanf(fdat,'%i',1);
        j(k) = fscanf(fdat,'%i',1);
        v(k) = fscanf(fdat,'%e',1);
      end
      AI = sparse(i,j,v,n,n);
    else
      fprintf ('\n### nnz (=%i) for AI should be >= 0\n\n',nnz);
      return
    end
    fgets(fdat);				% next line

    [i,j,v] = find(sparse(AI));			% don't trust the declared sparcity of AI
    nnz = length(i);

    if nnz < 0.8*mi*n

      fwrite(fbin,nnz,'int32');
      fprintf(ftxt,'%0i		nonzero elements in the sparse Jacobian of the inequality constraints, given below by their (row, col, val)\n',nnz);

      for k = 1:nnz
        fwrite(fbin,i(k),'int32');
        fwrite(fbin,j(k),'int32');
        fwrite(fbin,v(k),'double');
        fprintf(ftxt,'%i  %i  %23.16e\n',i(k),j(k),v(k));
      end

    else

      fwrite(fbin,-1,'int32');
      fprintf(ftxt,'%0i		the Jacobian of the inequality constraints is a dense matrix, with the following elements\n',-1);

      fwrite(fbin,AI,'double');
      fprintf(ftxt,'%23.16e\n',AI);

    end

    nlbi = sscanf(fgets(fdat),'%i',1);		% nb of lower inequality constraints
    fwrite(fbin,nlbi,'int32');
    fprintf(ftxt,'%0i		lower inequality constraints',nlbi);

    if nlbi
      fprintf(ftxt,', followed by their indices and values\n');
      for i = 1:nlbi
        j = fscanf(fdat,'%i',1);
        v = fscanf(fdat,'%e',1);
        fwrite(fbin,j,'int32');
        fwrite(fbin,v,'double');
        fprintf(ftxt,'%0i  %23.16e\n',j,v);
      end
      fgets(fdat);				% next line
    else
      fprintf(ftxt,'\n');
    end

    nubi = sscanf(fgets(fdat),'%i',1);		% nb of upper inequality constraints
    fwrite(fbin,nubi,'int32');
    fprintf(ftxt,'%0i		upper inequality constraints',nubi);

    if nubi
      fprintf(ftxt,', followed by their indices and values\n');
      for i = 1:nubi
        j = fscanf(fdat,'%i',1);
        v = fscanf(fdat,'%e',1);
        fwrite(fbin,j,'int32');
        fwrite(fbin,v,'double');
        fprintf(ftxt,'%0i  %23.16e\n',j,v);
      end
      fgets(fdat);				% next line
    else
      fprintf(ftxt,'\n');
    end

  end

  % Equality constraints
  % ^^^^^^^^^^^^^^^^^^^^

  me = fscanf(fdat,'%i',1);
  fwrite(fbin,me,'int32');
  fprintf(ftxt,'%0i		equality constraints\n',me);
  fgets(fdat);					% next line to skip the comment or to see whether the eof is reached

  if me

    nnz = sscanf(fgets(fdat),'%i',1);		% type of Jacobian
    if nnz == 0					% AE is full
      AE = fscanf(fdat,'%e',[me,n]);
    elseif nnz > 0				% AE is sparse
      for k = 1:nnz
        i(k) = fscanf(fdat,'%i',1);
        j(k) = fscanf(fdat,'%i',1);
        v(k) = fscanf(fdat,'%e',1);
      end
      AE = sparse(i,j,v,n,n);
    else
      fprintf ('\n### nnz (=%i) for AE should be >= 0\n\n',nnz);
      return
    end
    fgets(fdat);				% next line

    [i,j,v] = find(sparse(AE));			% don't trust the declared sparcity of AE
    nnz = length(i);

    if nnz < 0.8*me*n

      fwrite(fbin,nnz,'int32');
      fprintf(ftxt,'%0i		nonzero elements in the sparse Jacobian of the equality constraints, given below by their (row, col, val)\n',nnz);

      for k = 1:nnz
        fwrite(fbin,i(k),'int32');
        fwrite(fbin,j(k),'int32');
        fwrite(fbin,v(k),'double');
        fprintf(ftxt,'%i  %i  %23.16e\n',i(k),j(k),v(k));
      end

    else

      fwrite(fbin,-1,'int32');
      fprintf(ftxt,'%0i		the Jacobian of the equality constraints is a dense matrix, with the following elements\n',-1);

      fwrite(fbin,AE,'double');
      fprintf(ftxt,'%23.16e\n',AE);

    end

    be = fscanf(fdat,'%e',me);
    be = be(:);
    [i,j,v] = find(sparse(be));
    nnz = length(i);

    if nnz < 0.8*me

      fwrite(fbin,nnz,'int32');	% be is sparse
      fprintf(ftxt,'%0i		nonzero elements in the sparse right hand side, given below by their (index, value)\n',nnz);

      for k = 1:nnz
        fwrite(fbin,i(k),'int32');
        fwrite(fbin,v(k),'double');
        fprintf(ftxt,'%i  %23.16e\n',i(k),v(k));
      end

    else

      fwrite(fbin,-1,'int32');	% be is dense
      fprintf(ftxt,'%0i		the right hand side is dense, with the following elements\n',-1);

      fwrite(fbin,be,'double');	% 'real*8' in C or Fortran, floating point on 64 bits
      fprintf(ftxt,'%23.16e\n',be);

    end

    fgets(fdat);			% next line to see whether the eof is reached

  end

  % Solution
  % ^^^^^^^^

  if ~feof(fdat)

    xopt(1) = fscanf(fdat,'%e',1);	% primal solution
    fgets(fdat);			% next line to skip the comment
    if n > 1
      xopt(2:n) = fscanf(fdat,'%e',n-1)';
    end
    xopt = xopt(:);
    [i,j,v] = find(sparse(xopt));
    nnz = length(i);
    if nnz < 0.8*n
      fwrite(fbin,nnz,'int32');		% xopt is sparse
      fprintf(ftxt,'%0i		nonzero elements in the sparse primal solution, given below by their (index, value)\n',nnz);
      for k = 1:nnz
        fwrite(fbin,i(k),'int32');
        fwrite(fbin,v(k),'double');
        fprintf(ftxt,'%i  %23.16e\n',i(k),v(k));
      end
    else
      fwrite(fbin,-1,'int32');	% xopt is dense
      fprintf(ftxt,'%0i		the primal solution is dense with the following elements\n',-1);
      fwrite(fbin,xopt,'double');	% 'real*8' in C or Fortran, floating point on 64 bits
      fprintf(ftxt,'%23.16e\n',xopt);
    end

    lmopt(1) = fscanf(fdat,'%e',1);	% dual solution associated with the bound constraints
    fgets(fdat);			% next line to skip the comment
    if n > 1
      lmopt(2:n) = fscanf(fdat,'%e',n-1)';
    end
    lmopt = lmopt(:);
    [i,j,v] = find(sparse(lmopt));
    nnz = length(i);
    if nnz < 0.8*n
      fwrite(fbin,nnz,'int32');		% lmopt is sparse
      fprintf(ftxt,'%0i		nonzero elements in the sparse dual solution associated with the bound constraints, given below by their (index, value)\n',nnz);
      for k = 1:nnz
        fwrite(fbin,i(k),'int32');
        fwrite(fbin,v(k),'double');
        fprintf(ftxt,'%i  %23.16e\n',i(k),v(k));
      end
    else
      fwrite(fbin,-1,'int32');		% lmopt is dense
      fprintf(ftxt,'%0i		the dual solution associated with the bound constraints is dense with the following elements\n',-1);
      fwrite(fbin,lmopt,'double');	% 'real*8' in C or Fortran, floating point on 64 bits
      fprintf(ftxt,'%23.16e\n',lmopt);
    end

    if mi
      lmopt = fscanf(fdat,'%e',1);		% dual solution associated with the inequality constraints
      fgets(fdat);				% next line to skip the comment
      if mi > 1
        lmopt(2:mi) = fscanf(fdat,'%e',mi-1);
      end
      lmopt = lmopt(:);
      [i,j,v] = find(sparse(lmopt));
      nnz = length(i);
      if nnz < 0.8*mi
        fwrite(fbin,nnz,'int32');	% lmopt is sparse
        fprintf(ftxt,'%0i		nonzero elements in the sparse dual solution associated with the inequality constraints, given below by their (index, value)\n',nnz);
        for k = 1:nnz
          fwrite(fbin,i(k),'int32');
          fwrite(fbin,v(k),'double');
          fprintf(ftxt,'%i  %23.16e\n',i(k),v(k));
        end
      else
        fwrite(fbin,-1,'int32');	% lmopt is dense
        fprintf(ftxt,'%0i		the dual solution associated with the inequality constraints is dense with the following elements\n',-1);
        fwrite(fbin,lmopt,'double');	% 'real*8' in C or Fortran, floating point on 64 bits
        fprintf(ftxt,'%23.16e\n',lmopt);
      end
    end

    if me
      lmopt = fscanf(fdat,'%e',1);		% dual solution associated with the equality constraints
      fgets(fdat);				% next line to skip the comment
      if me > 1
        lmopt(2:me) = fscanf(fdat,'%e',me-1);
      end
      lmopt = lmopt(:);
      [i,j,v] = find(sparse(lmopt));
      nnz = length(i);
      if nnz < 0.8*me
        fwrite(fbin,nnz,'int32');	% lmopt is sparse
        fprintf(ftxt,'%0i		nonzero elements in the sparse dual solution associated with the equality constraints, given below by their (index, value)\n',nnz);
        for k = 1:nnz
          fwrite(fbin,i(k),'int32');
          fwrite(fbin,v(k),'double');
          fprintf(ftxt,'%i  %23.16e\n',i(k),v(k));
        end
      else
        fwrite(fbin,-1,'int32');	% lmopt is dense
        fprintf(ftxt,'%0i		the dual solution associated with the equality constraints is dense with the following elements\n',-1);
        fwrite(fbin,lmopt,'double');	% 'real*8' in C or Fortran, floating point on 64 bits
        fprintf(ftxt,'%23.16e\n',lmopt);
      end
    end

  end

  fclose(fdat);

  return
